@@ -1,6 +1,6 @@
 package Template::Provider::PerContextDBIC;
 # ABSTRACT: Load templates using DBIx::Class with per-context resultsets
-$Template::Provider::PerContextDBIC::VERSION = '0.000001';
+$Template::Provider::PerContextDBIC::VERSION = '0.000002';
 
 use strict;
 use warnings;
@@ -135,7 +135,7 @@ sub lookup_name {
 sub fetch {
 	my ( $self, $name ) = @_;
 	my $stat_ttl = $self->{STAT_TTL};
-# 	$self->debug("fetch($name)") if $self->{ DEBUG };
+	$self->debug("fetch($name)") if $self->{ DEBUG };
 
 	# We're not interested in GLOBs or file handles.
 	if ( ref $name ) {
@@ -163,7 +163,7 @@ sub fetch {
 
 	# Determine the path this template would be cached to.
 	my $compiled_filename = $self->_compiled_filename( $self->{DSN} . "/$cache_name" );
-# 	$self->debug("compiled_filename: $compiled_filename") if $self->{ DEBUG };
+	$self->debug("compiled_filename: $compiled_filename") if $self->{ DEBUG };
 
 	my ( $data, $error, $slot );
 
@@ -173,23 +173,25 @@ sub fetch {
 
 	# Otherwise, see if we already know the template is not found
 	if ( my $last_stat_time = $self->{NOTFOUND}->{$cache_name} ) {
-		#$self->debug("In NOTFOUND: $cache_name") if $self->{ DEBUG };
+		$self->debug("In NOTFOUND: $cache_name") if $self->{ DEBUG };
 		my $expires_in = $last_stat_time + $stat_ttl - time;
 		if ( $expires_in > 0 ) {
 			$self->debug(" file [$cache_name] in negative cache.  Expires in $expires_in seconds")
 			  if $self->{DEBUG};
 			return ( undef, Template::Constants::STATUS_DECLINED );
 		} else {
-			delete $self->{NOTFOUND}->{$name};
+			$self->debug(" remove file [$cache_name] from negative cache.")
+			  if $self->{DEBUG};
+			delete $self->{NOTFOUND}->{$cache_name};
 		}
 	} ## end if ( my $last_stat_time = $self->...)
 
 	# If caching is enabled and an entry already exists, refresh its cache
 	# slot and extract the data...
-	if ( $caching && ( $slot = $self->{LOOKUP}->{$cache_name} ) ) {
+	if ( $caching && ( $slot = $self->{LOOKUP}->{$cache_name} ) && !$self->_modified( $cache_name ) ) {
 		( $data, $error ) = $self->_refresh($slot);
 		$data = $slot->[Template::Provider::DATA] unless $error;
-# 		$self->debug("fetch - lookup from memory") if $self->{ DEBUG };
+		$self->debug("fetch - lookup from memory") if $self->{ DEBUG };
 	}
 	# ...otherwise if this template has already been compiled and cached (but
 	# not by this object) try to load it from the disk, providing it hasn't
@@ -197,7 +199,8 @@ sub fetch {
 	elsif ($compiled_filename
 		&& -f $compiled_filename
 		&& !$self->_modified( $cache_name, ( stat(_) )[9] ) ) {
-# 		$self->debug("fetch - load from cache disk") if $self->{ DEBUG };
+		$self->debug("template modified: ".$self->_modified( $cache_name, ( stat(_) )[9] )) if $self->{ DEBUG };
+		$self->debug("fetch - load from cache disk") if $self->{ DEBUG };
 		$data = $self->_load_compiled($compiled_filename);
 		$error = $self->error() unless $data;
 
@@ -207,13 +210,17 @@ sub fetch {
 	# ...else there is nothing already cached for this template so load it
 	# from the database.
 	else {
-# 		$self->debug("fetch - lookup from database") if $self->{ DEBUG };
+		$self->debug("fetch - lookup from database") if $self->{ DEBUG };
 		( $data, $error ) = $self->_load("$name");
 
 		if ($error) {
 			# Template could not be fetched.  Add to the negative/notfound cache.
-			#$self->debug("Adding to NOTFOUND: $cache_name") if $self->{ DEBUG };
+			$self->debug("Adding to NOTFOUND: $cache_name") if $self->{ DEBUG };
 			$self->{NOTFOUND}->{$cache_name} = time;
+			if ($compiled_filename && -f $compiled_filename) {
+				$self->debug("Removing cache file: $compiled_filename") if $self->{ DEBUG };
+				unlink $compiled_filename;
+			}
 		}
 
 		if ( !$error ) {
@@ -329,8 +336,9 @@ sub _modified {
 	# Try to retrieve the template from the database.
 	my ( $template, $tmpl_error ) = $self->get_template($lookup_name, qw/COLUMN_MODIFIED/);
 
-	return 0 if $tmpl_error;
-	return 0 unless $template;
+	# return true to say template is modified (must be true if template doesn't exist)
+	return 1 if $tmpl_error;
+	return 1 unless $template;
 
 	my $modified = Date::Parse::str2time( $template->get_column( $self->{COLUMN_MODIFIED} ) )
 	  || return $time ? 1 : 0;
@@ -354,7 +362,7 @@ Template::Provider::PerContextDBIC - Load templates using DBIx::Class with per-c
 
 =head1 VERSION
 
-version 0.000001
+version 0.000002
 
 =head1 SYNOPSIS
 
